@@ -15,13 +15,27 @@ irace.reload.debug <- function(package = "irace")
   paste("An unexpected condition occurred.",
         "Please report this bug to the authors of the irace package <http://iridia.ulb.ac.be/irace>")
 
+irace.print.memUsed <- function(objects)
+{
+  envir <- parent.frame()
+  if (missing(objects)) {
+    objects <- ls(envir = envir, all.names = TRUE)
+  }
+  x <- sapply(objects, function(name)
+              object.size(get(name, envir = envir)) / 1024)
+  # Do not print anything that is smaller than 10 Kb
+  x <- x[x > 10]
+  cat(sep="", sprintf("%30s : %17.1f Kb\n", names(x), x))
+  cat(sep="", sprintf("%30s : %17.1f Mb\n", "Total", sum(x) / 1024))
+}
+
 # Print a user-level fatal error message, when the calling context
 # cannot help the user to understand why the program failed.
 ## FIXME: rename this function to irace.error
 tunerError <- function(...)
 {
-  # The default is only 1000, which is too small. 8170 was the maximum
-  # value allowed in R 2.14.1
+  # The default is only 1000, which is too small. 8170 is the maximum
+  # value allowed up to R 3.0.2
   op <- options(warning.length = 8170)
   on.exit(options(op))
   stop (.irace.prefix, ..., call. = FALSE)
@@ -34,10 +48,19 @@ irace.assert <- function(exp)
   msg <- paste(deparse(mc), " is not TRUE\n", .irace.bug.report, sep = "")
   # FIXME: It would be great if we could save into a file the state of
   # the function that called this one.
+  print(traceback())
+  op <- options(warning.length = 8170, error = utils::recover)
+  on.exit(options(op))
   stop (msg)
   invisible()
 }
 
+irace.note <- function(...)
+{
+  cat ("# ", format(Sys.time(), usetz=TRUE), ": ",
+       paste(..., sep = "", collapse = ""), sep = "")
+}
+
 file.check <- function (file, executable = FALSE, readable = executable,
                         isdir = FALSE, notempty = FALSE, text = NULL)
 {
@@ -81,7 +104,9 @@ file.check <- function (file, executable = FALSE, readable = executable,
 }
 
 is.wholenumber <- function(x, tol = .Machine$double.eps^0.5)
-  { abs(x - round(x)) < tol }
+{
+  abs(x - round(x)) < tol
+}
 
 is.null.or.na <- function(x)
 {
@@ -94,7 +119,9 @@ is.null.or.empty <- function(x)
 }
 
 strcat <- function(..., collapse = NULL)
+{
   do.call(paste, args=list(..., sep="", collapse = collapse))
+}
 
 # FIXME: Isn't a R function to do this? More portable?
 canonical.dirname <- function(dirname = stop("required parameter"))
@@ -316,7 +343,7 @@ candidates.print.command <- function(cand, parameters)
   rownames(cand) <- cand$.ID.
   cand <- removeCandidatesMetaData(cand)
   cand <- cand[, unlist(parameters$names), drop = FALSE]
-  # A better way to do this? We cannot use apply() because the coerces
+  # A better way to do this? We cannot use apply() because that coerces
   # to a character matrix thus messing up numerical values.
   for(i in 1:nrow(cand)) {
     cat(sprintf("%-*d %s\n", nchar(nrow(cand)), i,
@@ -332,27 +359,36 @@ candidates.print.command <- function(cand, parameters)
 mpiInit <- function(nslaves, debugLevel = 0)
 {
   # Load the Rmpi package if it is not already loaded.
-  if (!is.loaded("mpi_initialize")) {
-    if (! require("Rmpi", quietly = TRUE))
-      tunerError("The `Rmpi' package is required for using MPI.")
-
-    # FIXME: We should do this when irace finalizes.
+  if (! ("Rmpi" %in% loadedNamespaces())) {
+    if (! suppressPackageStartupMessages
+        (requireNamespace("Rmpi", quietly = TRUE)))
+      tunerError("The 'Rmpi' package is required for using MPI")
+    
     # When R exits, finalize MPI.
     reg.finalizer(environment(Rmpi::mpi.exit), function(e) {
       # Rmpi already prints a message, so we don't need this.
       # cat("# Finalize MPI...\n")
       if (Rmpi::mpi.comm.size(1) > 0)
         # FIXME: dellog == TRUE tries to delete log files, but it does
-        # not take into account that we may have change directory and
-        # it does not fails gracefully but produces an annoying:
+        # not take into account that we may have changed directory and
+        # it does not fail gracefully but produces an annoying:
         # Warning message: running command 'ls *.30577+1.*.log 2>/dev/null' had status 2
         Rmpi::mpi.close.Rslaves(dellog = FALSE)
-      # FIXME: How to avoid the message
-      # "Rmpi cannot be used unless relaunching R" ?
-      Rmpi::mpi.exit()
+      # We would like to use .Call("mpi_finalize", PACKAGE = "Rmpi"), which is
+      # what mpi.finalize does, minus the annoying message: "Exiting Rmpi. Rmpi
+      # cannot be used unless relaunching R", which we do not care about
+      # because this finalizer should only be called when exiting R.
+      capture.output(Rmpi::mpi.finalize(),
+                     file = if (.Platform$OS.type == 'windows') 'NUL' else '/dev/null')
     }, onexit = TRUE)
 
     # Create slaves
-    Rmpi::mpi.spawn.Rslaves(nslaves = nslaves, quiet = (debugLevel == 0))
+    #  needlog: a logical. If TRUE, R BATCH outputs will be saved in log
+    #           files.  If FALSE, the outputs will send to /dev/null.
+    #  quiet: a logical. If TRUE, do not print anything unless an error occurs.
+    #         If FALSE, prints to stdio how many slaves are successfully
+    #         spawned and where they are running.
+    Rmpi::mpi.spawn.Rslaves(nslaves = nslaves, quiet = (debugLevel < 2),
+                            needlog = (debugLevel > 0))
   }
 }
